import { default as childProcess } from "child_process";
import { default as pathToFfmpeg } from "ffmpeg-static";
import makeDebugger from "debug";
const debug = makeDebugger("nyats:ffmpeg_extractor");
const ffmpeg_debug = makeDebugger("nyats:ffmpeg");
const ffmpegCommonParams = [
    "-c:v",
    "libwebp",
    "-f",
    "image2",
    "-",
    "-loglevel",
    "info",
    "-hide_banner",
];
const ffmpegTimeout = 60000; // 60s default timeout
function enableDebugging(process) {
    // Debug log stdout closing
    process.stdout.on("close", () => {
        debug("stdout closed");
    });
    // Pipe stderr to debug
    const stderr = [];
    process.stderr.on("data", (data) => {
        const strData = data.toString("utf8");
        stderr.push(strData);
        ffmpeg_debug(strData);
    });
    process.once("exit", (code) => {
        if (code !== 0) {
            const output = stderr.join();
            throw new Error(`ffmpeg exited with code ${code}: ${output}`, {
                cause: output,
            });
        }
    });
    process.once("error", (err) => {
        throw err;
    });
}
function spawnFfmpeg(params) {
    return childProcess.spawn(pathToFfmpeg, params, {
        windowsHide: true,
        timeout: ffmpegTimeout,
    });
}
async function ffmpegExtractor(params) {
    const ffmpegParams = params.concat(ffmpegCommonParams);
    ffmpeg_debug("ffmpeg ", ffmpegParams.join(" "));
    const ffmpeg = spawnFfmpeg(ffmpegParams);
    enableDebugging(ffmpeg);
    return new Promise((resolve) => {
        ffmpeg.once("spawn", () => {
            resolve(ffmpeg.stdout);
        });
    });
}
function scaleFilter(width, height) {
    return `scale=${width}:${height}:force_original_aspect_ratio=increase,crop=${width}:${height}`;
}
export const extractKeyFrames = async function (url, width, height) {
    debug("Extract first 40% scene change 30 vframes(full frames).");
    return ffmpegExtractor([
        "-skip_frame",
        "nointra",
        "-t",
        "600",
        "-i",
        `async:${url}`,
        "-an",
        "-vf",
        `${scaleFilter(width, height)}, setpts=N/(2*TB)`,
        // https://superuser.com/a/1704315
        "-compression_level",
        "6",
        "-q:v",
        "75",
        "-loop",
        "0",
        "-frames:v",
        "20",
    ]);
};
export const extractFirstFrame = async function (url, width, height) {
    debug("Extracting first frame");
    return ffmpegExtractor([
        "-i",
        `async:${url}`,
        "-vf",
        scaleFilter(width, height),
        "-frames:v",
        "1",
        "-vsync",
        "vfr",
    ]);
};
export const extractCoverArt = async function (url, width, height) {
    debug("Extract cover art for streams with attached pictures, thumbnails or cover art.");
    return ffmpegExtractor([
        "-i",
        `async:${url}`,
        "-map 0:v",
        "-map -0:V",
        "-vf",
        scaleFilter(width, height),
        "-frames:v 1",
    ]);
};
//# sourceMappingURL=ffmpeg_extractor.js.map